/*****************************/
/* kChat v1.7		     */
/* krzak@linux.net.pl        */ 
/* Marcin Krzyzanowski       */
/* http://krzak.linux.net.pl */
/* License: GPL              */
/*****************************/
/* Copyright (C) 2001  Marcin Krzyzanowski <krzak@linux.net.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.lang.*;
import java.net.*;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;

public class kChatChannel extends Component implements MouseListener {
    private int MaxLines = 100;
    private int pozX = 10;
    private int sizeX = 10;
    private int sizeY = 10;
    private int fontH = 0;
    private int startAt = 0;
    private static int len = 5;
    private Graphics buforGraf;
    private Image bufor;
    private Dimension rozmiarBuf,d;
    private Image bgImage = null;
    private int bgImageX = 0;
    private int bgImageY = 0;
    public FontMetrics fm = null;
    public Scrollbar sbar = null;
    boolean moderatorMode = false;
    private MessagesClass messages[] = new MessagesClass[MaxLines];
    private int messagesCount = 0;
    private boolean pressedM = false;

    public kChatChannel() {
    this.sizeX = 100;
    this.sizeY = 100;
    setSize(sizeX,sizeY);
    this.d = new Dimension(sizeX,sizeY);
    if (moderatorMode) addMouseListener(this);
    }
    

    public kChatChannel(int x, int y) {
    this.sizeX = x;
    this.sizeY = y;
    setSize(x,y);
    this.d = new Dimension(sizeX,sizeY);
    if (moderatorMode) addMouseListener(this);
    }
    
    public void setBackgroundImage(Image im)  { this.bgImage = im; }
    public void setScroller(Scrollbar sb) { this.sbar = sb; }

    public void moveStart(int pos) {
        startAt = pos;
	if (startAt<=0) startAt=0;
	if (startAt>=MaxLines) startAt=MaxLines-1;
        this.paint(this.getGraphics());
    }

    public void appendText(String text, String color) {
    // if end of messages[]
    if (messagesCount>=messages.length) {
	messagesCount--;
	for (int i=0;i<messages.length;i++) {
	    if ((i+1)<messages.length)  { MessagesClass old = messages[i+1];
			    messages[i] = old;
			    }
	    }
	}
    messages[messagesCount] = new MessagesClass();
    messages[messagesCount].text = text;
    messages[messagesCount].color =
 stringToColor(color);
    messages[messagesCount].lines = 0;

    FontMetrics fm = this.getGraphics().getFontMetrics(getFont());

    if ((fm.stringWidth(text) + fm.getMaxAdvance())>(sizeX-pozX)) {
	int w = 0; int j=0; int i =0;
	for (i=0; i<text.length();i++) { 
		w = w + fm.charWidth(text.charAt(i));
		if ((w+fm.getMaxAdvance())>(sizeX-pozX)) { 
		    messages[messagesCount].linesV.addElement(text.substring(j,i-1));
		    w = 0; j=i; 
		    messages[messagesCount].lines++;
		}
	}
	messages[messagesCount].linesV.addElement(text.substring(j-1,i));
	messages[messagesCount].lines++;
    } else  { 
    messages[messagesCount].linesV.addElement(text); 
    messages[messagesCount].lines=1; 
    }

   messagesCount++;
   this.paint(this.getGraphics());
   }

   public void update(Graphics g) {
   int pozY = 0;
   if (moderatorMode) pozX = 40;
    
    if ((buforGraf == null)  || (d.width != rozmiarBuf.width)  || (d.height != rozmiarBuf.height)) {
	rozmiarBuf = d;
	bufor = createImage(d.width,d.height);
	buforGraf = bufor.getGraphics();
	}    

    buforGraf.setFont(getFont());
    buforGraf.setColor(getBackground());
    buforGraf.fillRect(0, 0, d.width, d.height);
    if (bgImage != null) buforGraf.drawImage(bgImage,bgImageX,bgImageY,this);
    if (moderatorMode) addMouseListener(this);

    FontMetrics fm = getFontMetrics(getFont());
    fontH = fm.getHeight();
    len = (this.sizeY / fontH);

//  paint "len" last lines, start at "startAt", end at "endAt"
    pozY = (fontH * len);
    int line=0;
    int endAt=startAt+len;
    for(int i=(messagesCount-1);i>=0;i--) {
     if (messages[i] != null) {
	buforGraf.setColor(messages[i].color);
	for (int l=(messages[i].lines-1);l>=0;l--) {
	    if ((line) >= (len+startAt)) break;
	    if ((line>=startAt) && (line<=endAt)) {
		buforGraf.drawString((String)messages[i].linesV.elementAt(l),pozX,pozY); 
/* moderator */	if (moderatorMode) {
		    if (l==0) { 
			messages[i].button = new kChatButton();
			messages[i].button.drawButton(buforGraf,0,pozY-fontH,39,fontH,(String)messages[i].linesV.elementAt(l),i);
			}
		    }
		pozY=pozY-fontH;
		}
	    line++;
	    }
	if ((line) >= (len+startAt)) break;
      }
     }
    g.drawImage(bufor, 0, 0, this);
    }


    public Color stringToColor(String color)
    {
    Color c = Color.white;
    if (color.equals("white")) { c = Color.white.darker(); }
    if (color.equals("yellow")) { c = Color.yellow.darker().darker(); }
    if (color.equals("red")) {  c = Color.red.darker(); }
    if (color.equals("black")) { c = Color.black; }
    if (color.equals("green")) { c = Color.green.darker(); }
    if (color.equals("gray")) {  c = Color.darkGray; }
    return(c);
    }

    public String colorToString(Color color)
    {
    String s = "";
    if (color.equals(Color.white.darker())) { s = "white"; }
    if (color.equals(Color.yellow.darker().darker())) { s = "yellow"; }
    if (color.equals(Color.red.darker())) { s = "red"; }
    if (color.equals(Color.black)) { s = "black"; }
    if (color.equals(Color.green.darker())) { s = "green"; }
    if (color.equals(Color.darkGray)) { s = "gray"; }
    return(s);
    }

    public void paint(Graphics g) { update(g); }
    public Dimension getPreferredSize() {
 return getSize(); 
}

    public void mouseEntered(MouseEvent event) {
    }
    public void mouseExited(MouseEvent event) {
    }
    public void mousePressed(MouseEvent event) {
    }
    public void mouseReleased(MouseEvent event) {
    // workaround for to fast emited mouseClicked event (is it java(windows) bug??)
    pressedM = false;
    }
    public void mouseClicked(MouseEvent event) {
	if ((moderatorMode) && (pressedM == false)) {
	   pressedM = true;
	    for (int i=0;i<(messagesCount);i++) {
		// red box
		if ((messages[i] != null) && (messages[i].button != null ) &&
		    (event.getX()>messages[i].button.x1) && (event.getX()<messages[i].button.x2) && 
		    (event.getY()>messages[i].button.y1) && (event.getY()<messages[i].button.y2))
		    { 
		    messages[i]=null; 
		    }
		// green box
		if ((messages[i] != null) && (messages[i].button != null ) &&
		    (event.getX()>messages[i].button.x3) && (event.getX()<messages[i].button.x4) && 
		    (event.getY()>messages[i].button.y3) && (event.getY()<messages[i].button.y4))
		    { 
		    kChatApplet.net.send("*MSG|" + kChatApplet.login + "|CHAN|" + messages[i].text + "|" + colorToString(messages[i].color) + "|"); 
		    }
	    }
	this.paint(this.getGraphics());
	}
    pressedM = true;
    }

}

class MessagesClass {
    public String text = new String();
    public Color color = Color.black;
    public Vector linesV = new Vector();
    public kChatButton button = null;
    public int lines = 1;
}