/*****************************/
/* kChat v1.7		     */
/* krzak@linux.net.pl        */ 
/* Marcin Krzyzanowski       */
/* http://krzak.linux.net.pl */
/* License: GPL              */
/*****************************/
/* Copyright (C) 2001  Marcin Krzyzanowski <krzak@linux.net.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
import java.io.*;
import java.net.*;

public class kChatNet extends Thread implements Runnable {
    /** Are we connected or not? */
    public boolean connected;
    private Socket socket;
    private Thread thread;
    private BufferedWriter os;
    private BufferedReader is;
    private int port = 1979;
    
    /** Start a thread. */
    public void start() {
	if (thread == null) {
	    clear();
		 
	    thread = new Thread(this);
	    thread.start();
	}
    }
   
    /** The main loop. */
    public void run() {
	String buf;
	  	  
	while (Thread.currentThread() == thread) {
	    try {
		buf = is.readLine();
			
		if (buf == null)
		    return;
			
		 kChatProtocol.parse(buf);
	    } catch (IOException e) {
		return;
	    }
	}		
    }
   
    /** Connect to the server. */
    public synchronized void connect(String host) {
	if (connected)
	    return;
	  		
	try {
	    socket = new Socket(host, port);		   
	} catch (UnknownHostException e) {
	    return;
	} catch (IOException e) {
	    return;
	}
	  
	try {
	    os = new BufferedWriter(new
		OutputStreamWriter(socket.getOutputStream()));
	    is = new BufferedReader(new
		InputStreamReader(socket.getInputStream()));
	} catch (IOException e) {
	    return;
	}
	  
	if (os != null && is != null) {
	    connected = true;
	    notify();
		 
	    if (thread == null)
		start();
	}
    }
   
    /** Wait until we have connected. */
    private void waitConnect() {
	kChatApplet.chanWnd.appendText("Trwa laczenie....","black");
	while (!connected) {
	    try {
		wait();
	    } catch (InterruptedException e) {
		kChatApplet.chanWnd.appendText("Nie ma polaczenia z serwerem","black");
		return;
	    }
	}
    }
   
    /** Send a message to the server. */
    public void send(String data) {
	try {
	    data.trim();
	    os.write(data + "\r\n");
	    os.flush();
	} catch (IOException e) {
	    kChatApplet.chanWnd.appendText("Nie ma polaczenia z serwerem","black");
	    return;
	} catch (NullPointerException e) {
	}
    }
   
    /** Disconnect. */
    public synchronized void disconnect() {
//	send("CLOSE");
	connected = false;
	os = null;
	is = null;
	thread.stop();
	thread = null;
	try {
	socket.close();
	} catch (IOException e) {
	}
  
}

    /** Clear variables. */
    public void clear() {
    }
}
